@@ -75,13 +75,14 @@ class Displayable(renpy.object.Object):
     their fields.
     """
 
+    activated = False
     focusable = False
     full_focus_name = None
     role = ''
 
     def __init__(self, focus=None, default=False, style='default', **properties):
-        self.style = renpy.style.Style(style, properties)
-        self.style_prefix = 'insensitive_'
+        self.style = renpy.style.Style(style, properties, heavy=True)
+        # self.style_prefix = 'insensitive_'
         self.focus_name = focus
         self.default = default
 
@@ -106,9 +107,10 @@ class Displayable(renpy.object.Object):
         Called to indicate that this widget has the focus.
         """
 
-        self.set_style_prefix(self.role + "hover_")
+        if not self.activated:
+            self.set_style_prefix(self.role + "hover_")
         
-        if not default:
+        if not default and not self.activated:
             renpy.audio.sound.play(self.style.sound)
 
     def unfocus(self):
@@ -116,7 +118,8 @@ class Displayable(renpy.object.Object):
         Called to indicate that this widget has become unfocused.
         """
 
-        self.set_style_prefix(self.role + "idle_")
+        if not self.activated:
+            self.set_style_prefix(self.role + "idle_")
 
 
 
@@ -129,11 +132,13 @@ class Displayable(renpy.object.Object):
         widgets, if any.
         """
 
-        if prefix == self.style_prefix:
+        if prefix == self.style.prefix:
             return
+        
+        # if prefix == self.style_prefix:
+        #    return
 
         self.style.set_prefix(prefix)
-        self.style_prefix = prefix
         renpy.display.render.redraw(self, 0)
 
     def parameterize(self, name, parameters):
@@ -184,7 +189,9 @@ class Displayable(renpy.object.Object):
         to return something more sensible.
         """
 
-        return self.style.xpos, self.style.ypos, self.style.xanchor, self.style.yanchor
+        return (self.style.xpos, self.style.ypos,
+                self.style.xanchor, self.style.yanchor,
+                self.style.xoffset, self.style.yoffset)
 
     def visit_all(self, callback):
         """
@@ -236,7 +243,7 @@ class Displayable(renpy.object.Object):
 
         return
 
-    def place(self, dest, x, y, width, height, surf, xoff=None, yoff=None):
+    def place(self, dest, x, y, width, height, surf, xoff=None, yoff=None, main=True):
         """
         This draws this Displayable onto a destination surface, using
         the placement style information returned by this object's
@@ -261,7 +268,7 @@ class Displayable(renpy.object.Object):
         self.render().
         """
 
-        xpos, ypos, xanchor, yanchor = self.get_placement()
+        xpos, ypos, xanchor, yanchor, xoffset, yoffset = self.get_placement()
 
         if xpos is None:
             xpos = 0
@@ -325,38 +332,95 @@ class Displayable(renpy.object.Object):
 
         yoff += y
 
-        dest.blit(surf, (xoff, yoff))
+        # Add in offsets.
+        xoff += xoffset
+        yoff += yoffset
+        
+        dest.blit(surf, (xoff, yoff), main=main)
 
         return xoff, yoff
 
+class ImagePredictInfo(renpy.object.Object):
+    """
+    This stores information involved in image prediction.
+    """
 
-class SceneLists(object):
+    def after_setstate(self):
+        for i in renpy.config.layers + renpy.config.top_layers:
+            self.images.setdefault(i, {})
+    
+    def __init__(self, ipi=None):
+
+        # layer -> (tag -> image name)
+        self.images = { }
+
+        if ipi is None:
+            for i in renpy.config.layers + renpy.config.top_layers:
+                self.images[i] = { }
+        else:
+            for i in renpy.config.layers + renpy.config.top_layers:
+                self.images[i] = ipi.images[i].copy()
+            
+                
+    def showing(self, layer, name):
+
+        shown = self.images[layer].get(name[0], None)
+        
+        if shown is None or len(shown) < len(name):
+            return False
+
+        for a, b in zip(name, shown):
+            if a != b:
+                return False
+
+        return True
+
+    def predict_scene(self, layer):
+        self.images[layer].clear()
+
+    def predict_show(self, name, layer):
+        self.images[layer][name[0]] = name
+        
+    def predict_hide(self, tag, layer):
+        self.images[layer].pop(tag, None)
+    
+
+class SceneLists(renpy.object.Object):
     """
     This stores the current scene lists that are being used to display
     things to the user. 
     """
 
-
-    def __setstate__(self, state):
-
-        self.__dict__.update(state)
-
+    __version__ = 1
+    
+    def after_setstate(self):
         for i in renpy.config.layers + renpy.config.top_layers:
             if i not in self.layers:
                 self.layers[i] = [ ]
+                self.at_list[i] = { }
 
+    def after_upgrade(self, version):
 
-    def __init__(self, oldsl=None):
+        if version < 1:
 
+            self.at_list = { }
+            self.layer_at_list = { }
 
+            for i in renpy.config.layers + renpy.config.top_layers:
+                self.at_list[i] = { }
+                self.layer_at_list[i] = (None, [ ])
+                
+
+    def __init__(self, oldsl, ipi):
 
         # A map from layer name -> list of
         # (key, zorder, show time, animation time, displayable) 
-        #
-        # The structure of a scene list is used in:
-        # Scenelists, Interface.make_layer, Fixed, and MoveTransition.
         self.layers = { }
-         
+        self.at_list = { }
+        self.layer_at_list = { }
+        
+        self.image_predict_info = ipi
+        
         if oldsl:
 
             for i in renpy.config.layers + renpy.config.top_layers:
@@ -365,42 +429,27 @@ class SceneLists(object):
                 except KeyError:
                     self.layers[i] = [ ]
 
+                self.at_list[i] = oldsl.at_list[i].copy()
+                self.layer_at_list[i] = oldsl.layer_at_list[i]
+                
             for i in renpy.config.overlay_layers:
                 self.clear(i)
 
             self.replace_transient()
 
-            self.sticky_positions = oldsl.sticky_positions.copy()
             self.movie = oldsl.movie
             self.focused = None
             
         else:
             for i in renpy.config.layers + renpy.config.top_layers:
                 self.layers[i] = [ ]
-
+                self.at_list[i] = { }
+                self.layer_at_list[i] = (None, [ ])
+                
             self.music = None
             self.movie = None
-            self.sticky_positions = { }
             self.focused = None
 
-    def rollback_copy(self):
-        """
-        Makes a shallow copy of all the lists, except for the stack,
-        which is a 2-level copy.
-        """
-
-        rv = SceneLists(self)
-        rv.focused = None
-
-#         rv.master = self.master[:]
-#         rv.transient = self.transient[:]
-
-#         rv.music = self.music
-#         rv.movie = self.movie
-
-        return rv
-         
-
     def replace_transient(self):
         """
         Replaces the contents of the transient display list with
@@ -411,7 +460,10 @@ class SceneLists(object):
 
         for i in renpy.config.transient_layers:
             self.layers[i] = [ ]
-
+            self.at_list[i].clear()
+            self.image_predict_info.images[i].clear()
+            self.layer_at_list[i] = (None, [ ])
+            
     def transient_is_empty(self):
         """
         This returns True if all transient layers are empty. This is
@@ -427,7 +479,7 @@ class SceneLists(object):
 
         return True
 
-    def add(self, layer, thing, key=None, zorder=0, behind=[ ]):
+    def add(self, layer, thing, key=None, zorder=0, behind=[ ], at_list=[ ], name=None):
         """
         This is called to add something to a layer. Layer is
         the name of the layer that we need to add the thing to,
@@ -452,6 +504,12 @@ class SceneLists(object):
         if layer not in self.layers:
             raise Exception("Trying to add something to non-existent layer '%s'." % layer)
 
+        if key:
+            self.at_list[layer][key] = at_list
+
+        if key and name:
+            self.image_predict_info.images[layer][key] = name
+                
         l = self.layers[layer]
 
         at = None
@@ -498,12 +556,15 @@ class SceneLists(object):
 
         if layer not in self.layers:
             raise Exception("Trying to remove something from non-existent layer '%s'." % layer)
-
+        
         l = self.layers[layer]
         l = [ (k, zo, st, at, d) for k, zo, st, at, d in l if k != thing if d is not thing ]
-
         self.layers[layer] = l
 
+        self.at_list[layer].pop(thing, None)
+        self.image_predict_info.images[layer].pop(thing, None)
+            
+
     def clear(self, layer):
         """
         Clears the named layer, making it empty.
@@ -513,13 +574,22 @@ class SceneLists(object):
             raise Exception("Trying to clear non-existent layer '%s'." % layer)
 
         self.layers[layer] = [ ]
+        self.at_list[layer].clear()
+        self.image_predict_info.images[layer].clear()
+        self.layer_at_list[layer] = (None, [ ])
 
+    def set_layer_at_list(self, layer, at_list):
+        self.layer_at_list[layer] = (None, list(at_list))
+        
     def set_times(self, time):
         """
         This finds entries with a time of None, and replaces that
         time with the given time.
         """
 
+        for l, (t, list) in self.layer_at_list.items():
+            self.layer_at_list[l] = (t or time, list)
+        
         for l in self.layers.values():
             ll = [ ]
 
@@ -530,17 +600,34 @@ class SceneLists(object):
             l[:] = ll
             
 
-    def showing(self, layer, key):
+    def showing(self, layer, name):
         """
-        Returns true of tan entry with the given key is found in the
-        given layer. Returns False otherwise. Key should be a string.
+        Returns true if something with the prefix of the given name
+        is found in the scene list.
         """
 
-        for k, zo, st, at, d in self.layers[layer]:
-            if k == key:
-                return True
+        return self.image_predict_info.showing(layer, name)
 
-        return False
+    def make_layer(self, layer, properties):
+        """
+        Creates a Fixed with the given layer name and scene_list.
+        """
+
+        rv = renpy.display.layout.MultiBox(layout='fixed', focus=layer, **properties)
+        rv.append_scene_list(self.layers[layer])
+
+        time, at_list = self.layer_at_list[layer]
+
+        if at_list:
+            for a in at_list:
+                rv = a(rv)
+
+                f = renpy.display.layout.MultiBox(layout='fixed')
+                f.add(rv, time, time)
+                rv = f
+
+        rv.layer_name = layer
+        return rv
     
 
 class Display(object):
@@ -656,11 +743,16 @@ class Display(object):
         self.mouse_backing_pos = None
         self.mouse_info = None
 
-        self.suppress_mouse = False
+        self.mouse_event_time = get_time()
+        
+        self.suppressed_blit = False
         self.full_redraw = True
 
         self.next_frame = 0
 
+        # A tree of surfaces from the last time the screen was rendered.
+        self.surftree = None
+        
     def can_redraw(self, first_pass):
         """
         Uses the framerate to determine if we can and should redraw.
@@ -694,6 +786,11 @@ class Display(object):
 
         return True
 
+    def mouse_event(self, ev):
+
+        if ev.type == pygame.MOUSEMOTION or pygame.MOUSEBUTTONDOWN or pygame.MOUSEBUTTONUP:
+            self.mouse_event_time = get_time()
+         
     def show_mouse(self, pos, info):
         """
         Actually shows the mouse.
@@ -743,15 +840,29 @@ class Display(object):
         drawn, and only redraws if the mouse has actually been moved.
         """
 
+        # Figure out if the mouse visibility algorithm is hiding the mouse.
+        if self.mouse_event_time + renpy.config.mouse_hide_time < get_time():
+            visible = False
+        else:
+            visible = True
+
+        # Deal with a hardware mouse, the easy way.
         if not self.mouse:
+            pygame.mouse.set_visible(visible)
             return [ ]
 
-        if self.suppress_mouse:
+        # The rest of this is for the software mouse.
+        
+        if self.suppressed_blit:
             return [ ]
 
+        visible = show_mouse and visible
+        
+        mouse_kind = renpy.display.focus.get_mouse() or self.interface.mouse 
+        
         # Figure out the mouse animation.
-        if self.interface.mouse in renpy.config.mouse:
-            anim = renpy.config.mouse[self.interface.mouse]
+        if mouse_kind in renpy.config.mouse:
+            anim = renpy.config.mouse[mouse_kind]
         else:
             anim = renpy.config.mouse['default']
 
@@ -759,10 +870,10 @@ class Display(object):
 
         pos = pygame.mouse.get_pos()
 
-        if pos == self.mouse_location and show_mouse and info == self.mouse_info:
-            return [ ]
-
-        if not pos and not show_mouse:
+        if (pos == self.mouse_location and
+            show_mouse and
+            info == self.mouse_info):
+            
             return [ ]
 
         updates = [ ]
@@ -770,7 +881,7 @@ class Display(object):
         if self.mouse_location:
             updates.append(self.hide_mouse())
 
-        if show_mouse:
+        if visible and pos:
             updates.append(self.show_mouse(pos, info))
             
         return updates
@@ -779,7 +890,7 @@ class Display(object):
         """
         Draws the mouse, and then updates the screen.
         """
-
+        
         updates = self.draw_mouse()
 
         if updates:
@@ -826,9 +937,11 @@ class Display(object):
         else:
             self.full_redraw = True
             
-        self.suppress_mouse = suppress_blit
+        self.suppressed_blit = suppress_blit
 
         renpy.display.focus.take_focuses(surftree.focuses)
+
+        self.surftree = surftree
         
     def save_screenshot(self, filename):
         """
@@ -921,7 +1034,8 @@ class Interface(object):
         self.ticks = 0
         self.mouse = 'default'
         self.timeout_time = None
-
+        self.last_event = None
+        
         # Things to be preloaded.
         self.preloads = [ ]
 
@@ -990,7 +1104,7 @@ class Interface(object):
             return False
         else:
             self.set_transition(trans)
-            return self.interact(show_mouse=False, trans_pause=True,
+            return self.interact(trans_pause=True,
                                  suppress_overlay=not renpy.config.overlay_during_with,
                                  mouse='with')
 
@@ -1047,11 +1161,12 @@ class Interface(object):
         if self.pushed_event:
             rv = self.pushed_event
             self.pushed_event = None
-            return rv
-
         else:
-            return pygame.event.poll()
-        
+            rv = pygame.event.poll()
+
+        self.last_event = rv
+        return rv
+            
 
     def event_wait(self):
         """
@@ -1062,15 +1177,16 @@ class Interface(object):
         if self.pushed_event:
             rv = self.pushed_event
             self.pushed_event = None
+            self.last_event = rv
             return rv
 
         # We load at most one image per wait.
         if renpy.display.im.cache.needs_preload():
             ev = pygame.event.poll()
             if ev.type != NOEVENT:
+                self.last_event = ev
                 return ev
 
-
             renpy.display.im.cache.preload()
 
         try:
@@ -1079,18 +1195,9 @@ class Interface(object):
         finally:
             cpu_idle.clear()
 
+        self.last_event = ev
         return ev
 
-    def make_layer(self, name, scene_list):
-        """
-        Creates a Fixed with the given layer name and scene_list.
-        """
-
-        rv = renpy.display.layout.Fixed(focus=name, **self.layer_properties[name])
-        rv.append_scene_list(scene_list)
-        rv.layer_name = name
-        return rv
-    
 
     def compute_scene(self, scene_lists):
         """
@@ -1102,9 +1209,8 @@ class Interface(object):
 
         for layer in renpy.config.layers + renpy.config.top_layers:
 
-            f = self.make_layer(layer, scene_lists.layers[layer])
-            rv[layer] = f
-
+            rv[layer] = scene_lists.make_layer(layer, self.layer_properties[layer])
+            
         return rv
             
 
@@ -1220,7 +1326,7 @@ class Interface(object):
         renpy.ui.close()
 
         # The root widget of everything that is displayed on the screen.
-        root_widget = renpy.display.layout.Fixed() 
+        root_widget = renpy.display.layout.MultiBox(layout='fixed') 
         root_widget.layers = { }
 
         # A list of widgets that are roots of trees of widgets that are
@@ -1248,7 +1354,7 @@ class Interface(object):
                 w.predict(renpy.display.im.cache.get)
 
         # The root widget of all of the layers.
-        layers_root = renpy.display.layout.Fixed()
+        layers_root = renpy.display.layout.MultiBox(layout='fixed')
         layers_root.layers = { }
 
         def add_layer(where, layer):
@@ -1262,6 +1368,7 @@ class Interface(object):
 
                 trans = self.transition[layer](old_widget=self.old_scene[layer],
                                                new_widget=scene_layer)
+
                 where.add(trans)
                 where.layers[layer] = trans
                 
@@ -1279,16 +1386,16 @@ class Interface(object):
             not self.suppress_transition):
 
             # Compute what the old root should be.
-            old_root = renpy.display.layout.Fixed()
+            old_root = renpy.display.layout.MultiBox(layout='fixed')
             old_root.layers = { }
 
             for layer in renpy.config.layers:
                 old_root.layers[layer] = self.old_scene[layer]
                 old_root.add(self.old_scene[layer])
-            
+
             trans = self.transition[None](old_widget=old_root,
                                           new_widget=layers_root)
-            
+                
             root_widget.add(trans)
 
             if trans_pause:
@@ -1364,8 +1471,7 @@ class Interface(object):
 
                 # Redraw the screen.
                 if needs_redraw and self.display.can_redraw(first_pass):
-
-
+                    
                     # If we have a movie, start showing it.
                     suppress_blit = renpy.display.video.interact()
 
@@ -1388,6 +1494,11 @@ class Interface(object):
                     if first_pass:
                         scene_lists.set_times(self.interact_time)
 
+                    if first_pass and self.last_event:
+                        x, y = pygame.mouse.get_pos()
+                        x -= self.display.screen_xoffset
+                        renpy.display.focus.mouse_handler(self.last_event, x, y, default=False)
+
                     needs_redraw = False
                     first_pass = False
 
@@ -1434,6 +1545,7 @@ class Interface(object):
                     
                     # Handle the redraw timer.
                     redraw_time = renpy.display.render.redraw_time()
+
                     if redraw_time and not needs_redraw:
 
                         if redraw_time != old_redraw_time:
@@ -1513,7 +1625,12 @@ class Interface(object):
 
                         renpy.audio.audio.periodic()
                         continue
-                            
+
+                    
+                    # This checks the event to see if it's a mouse event,
+                    # and updates the mouse event timer as appropriate.
+                    self.display.mouse_event(ev)
+                    
                     # This can set the event to None, to ignore it.
                     ev = renpy.display.joystick.event(ev)
                     if not ev:
@@ -1545,6 +1662,8 @@ class Interface(object):
 
                     self.event_time = end_time = get_time()
 
+
+                    # Handle the event normally.
                     rv = renpy.display.focus.mouse_handler(ev, x, y)
                     
                     if rv is None:
@@ -1555,6 +1674,12 @@ class Interface(object):
 
                     if rv is not None:
                         break
+
+                    # Handle displayable inspector.
+                    if renpy.config.inspector and renpy.display.behavior.inspector(ev):
+                        l = self.display.surftree.main_displayables_at_point(x, y, renpy.config.transient_layers + renpy.config.overlay_layers)
+                        renpy.game.invoke_in_new_context(renpy.config.inspector, l)
+                        
             
                 except IgnoreEvent:
                     # An ignored event can change the timeout. So we want to
