@@ -540,11 +540,8 @@ class FrameImage(ImageBase):
 
                     surf2 = pygame.Surface(dstsize, 0, surf)
 
-                    print "---"
-
                     for y in range(0, dsth, tileh):
                         for x in range(0, dstw, tilew):
-                            print x, y
                             surf2.blit(surf, (x, y))
 
                     surf = surf2
@@ -845,6 +842,7 @@ class Twocolor(ImageBase):
     def predict_files(self):
         return self.image.predict_files()
 
+
 class Recolor(ImageBase):
     """
     This adjusts the colors of the image that is its child. It takes as an
@@ -887,6 +885,188 @@ class Recolor(ImageBase):
     def predict_files(self):
         return self.image.predict_files()
 
+class MatrixColor(ImageBase):
+    """
+    This applies a 20 element matrix, that turns the color vector:
+    [ r, g, b, a, 1 ] into [ r, g, b, a ].
+    """
+
+    def __init__(self, im, matrix, **properties):
+
+        im = image(im)
+
+        if len(matrix) != 20 and len(matrix) != 25:
+            raise Exception("ColorMatrix expects a 20 or 25 element matrix, got %d elements." % len(matrix))
+
+        matrix = tuple(matrix)        
+        super(MatrixColor, self).__init__(im, matrix, **properties)
+        
+        self.image = im
+        self.matrix = matrix
+        
+    def load(self):
+
+        surf = cache.get(self.image)
+
+        if not renpy.display.module.can_colormatrix:
+            return surf
+
+        if not (surf.get_masks()[3]):
+            surf = surf.convert_alpha()
+
+        rv = pygame.Surface(surf.get_size(), surf.get_flags(), surf)
+
+        renpy.display.module.colormatrix(surf, rv, self.matrix)
+        
+        return rv
+
+    def predict_files(self):
+        return self.image.predict_files()
+
+class matrix(tuple):
+    """
+    This class represents a 5x5 mathematical matrix.
+    """
+
+    def __new__(cls, *args):
+
+        if len(args) == 1:
+            args = tuple(args[0])
+
+        if len(args) == 20:
+            args = args + (0, 0, 0, 0, 1)
+
+        if len(args) != 25:
+            raise Exception("Matrix expects to be given 20 or 25 entries, not %d." % len(args))
+
+        return tuple.__new__(cls, args)
+    
+    def mul(self, a, b):
+
+        if not isinstance(a, matrix):
+            a = matrix(a)
+
+        if not isinstance(b, matrix):
+            b = matrix(b)
+            
+        result = [ 0 ] * 25
+        for y in range(0, 5):
+            for x in range(0, 5):
+                for i in range(0, 5):
+                    result[x + y * 5] += a[x + i * 5] * b[i + y * 5]
+                    
+        return matrix(result)
+
+    def scalar_mul(self, other):
+        other = float(other)
+        return matrix([ i * other for i in self ])
+
+    def __add__(self, other):
+        if isinstance(other, (int, float)):
+            other = float(other)
+            return matrix([ i + other for i in self ])
+
+        other = matrix(other)
+        return matrix([ i + j for i, j in zip(self, other)])
+
+    __radd__ = __add__
+
+    def __sub__(self, other):
+        return self + other * -1
+
+    def __rsub__(self, other):
+        return self * -1 + other
+        
+    def __mul__(self, other):
+        if isinstance(other, (int, float)):
+            return self.scalar_mul(other)
+
+        return self.mul(self, other)
+
+    def __rmul__(self, other):
+        if isinstance(other, (int, float)):
+            return self.scalar_mul(other)
+
+        return self.mul(other, self)
+
+    def __repr__(self):
+        return """\
+im.matrix(%f, %f, %f, %f, %f.
+          %f, %f, %f, %f, %f,
+          %f, %f, %f, %f, %f,
+          %f, %f, %f, %f, %f,
+          %f, %f, %f, %f, %f)""" % self
+
+    
+    def identity():
+        return matrix(1, 0, 0, 0, 0,
+                      0, 1, 0, 0, 0,
+                      0, 0, 1, 0, 0,
+                      0, 0, 0, 1, 0)
+
+    identity = staticmethod(identity)
+
+    def saturation(level, desat=(0.2126, 0.7152, 0.0722)):
+
+        r, g, b = desat
+        
+        def I(a, b):
+            return a + (b - a) * level
+
+        return matrix(I(r, 1), I(g, 0), I(b, 0), 0, 0,
+                      I(r, 0), I(g, 1), I(b, 0), 0, 0,
+                      I(r, 0), I(g, 0), I(b, 1), 0, 0,
+                      0, 0, 0, 1, 0)
+
+    saturation = staticmethod(saturation)
+    
+    def desaturate():
+        return matrix.saturation(0.0)
+    
+    desaturate = staticmethod(desaturate)
+
+    
+    def tint(r, g, b):
+        return matrix(r, 0, 0, 0, 0,
+                      0, g, 0, 0, 0,
+                      0, 0, b, 0, 0,
+                      0, 0, 0, 1, 0)
+
+    tint = staticmethod(tint)
+
+    def invert():
+        return matrix(-1, 0, 0, 0, 1,
+                      0, -1, 0, 0, 1,
+                      0, 0, -1, 0, 1,
+                      0, 0, 0, 1, 0)
+
+    invert = staticmethod(invert)
+
+    def brightness(b):
+        return matrix(1, 0, 0, 0, b,
+                      0, 1, 0, 0, b,
+                      0, 0, 1, 0, b,
+                      0, 0, 0, 1, 0)
+
+    brightness = staticmethod(brightness)
+
+    def opacity(o):
+        return matrix(1, 0, 0, 0, 0,
+                      0, 1, 0, 0, 0,
+                      0, 0, 1, 0, 0,
+                      0, 0, 0, o, 0)
+
+    opacity = staticmethod(opacity)
+
+    def contrast(c):
+        return matrix.brightness(-.5) * matrix.tint(c, c, c) * matrix.brightness(.5) 
+    
+def Grayscale(im, desat=(0.2126, 0.7152, 0.0722)):
+    return MatrixColor(im, matrix.saturation(0.0, desat))
+
+def Sepia(im, tint=(1.0, .94, .76), desat=(0.2126, 0.7152, 0.0722)):
+    return MatrixColor(im, matrix.saturation(0.0, desat) * matrix.tint(tint[0], tint[1], tint[2]))
+    
 def Color(im, color):
     """
     This recolors the supplied image, mapping colors such that black is
