@@ -99,6 +99,7 @@ class Container(renpy.display.core.Displayable):
         for i in self.children:
             i.set_style_prefix(prefix)
 
+            
     def add(self, child):
         """
         Adds a child to this container.
@@ -121,7 +122,7 @@ class Container(renpy.display.core.Displayable):
         return rv
 
     def event(self, ev, x, y, st):
-
+                
         children_offsets = zip(self.children, self.offsets)
         children_offsets.reverse()
 
@@ -157,103 +158,7 @@ class Container(renpy.display.core.Displayable):
     def visit(self):
         return self.children
     
-class Fixed(Container):
-    """
-    A container that lays out each of its children at fixed
-    coordinates determined by the position style of the child. Each
-    widget is given the whole area of this widget, and then placed
-    within that area based on its position style.
-
-    The result of this layout is the size of the entire area allocated
-    to it. So it's probably only viable for laying out a root window.
-
-    Fixed is used by the display core to render scene lists, and to
-    pass them off to transitions.
-
-    """
-
-    layer_name = None
     
-    def __init__(self, style='default', **properties):
-        super(Fixed, self).__init__(style=style, **properties)
-        self.start_times = [ ]
-        self.anim_times = [ ]
-
-        # A map from layer name to the widget corresponding to
-        # that layer.
-        self.layers = None
-
-        # The scene list for this widget.
-        self.scene_list = None
-        
-
-    def add(self, widget, start_time=None, anim_time=None):
-        super(Fixed, self).add(widget)
-        self.start_times.append(start_time)
-        self.anim_times.append(anim_time)
-
-    def append_scene_list(self, l):
-        for tag, zo, start, anim, d in l:
-            self.add(d, start, anim)
-
-        if self.scene_list is None:
-            self.scene_list = [ ]
-            
-        self.scene_list.extend(l)
-
-#     def get_widget_time_list(self):
-#         return zip(self.children, self.times)
-            
-    def render(self, width, height, st, at):
-
-        self.offsets = [ ]
-        self.sizes = [ ]
-
-        rv = renpy.display.render.Render(width, height)
-
-        t = renpy.game.interface.frame_time
-        it = renpy.game.interface.interact_time
-
-        # Things with a None time are started at the first draw.
-
-        self.start_times = [ i or it for i in self.start_times ]
-        self.anim_times = [ i or it for i in self.anim_times ]
-
-        
-        for child, start, anim in zip(self.children, self.start_times, self.anim_times):
-
-            cst = t - start
-            cat = t - anim
-
-            surf = render(child, width, height, cst, cat)
-
-            if surf:
-                self.sizes.append(surf.get_size())
-                offset = child.place(rv, 0, 0, width, height, surf)
-                self.offsets.append(offset)
-            else:
-                self.sizes.append((0, 0))
-                self.offsets.append((0, 0))
-
-        return rv
-
-    def event(self, ev, x, y, st):
-        children_offsets = zip(self.children, self.offsets, self.start_times)
-        children_offsets.reverse()
-
-        for i, (xo, yo), t in children_offsets: 
-
-            if t is None:
-                cst = 0
-            else:
-                cst = renpy.game.interface.event_time - t
-
-            rv = i.event(ev, x - xo, y - yo, cst)    
-            if rv is not None:
-                return rv
-                
-        return None
-
 
 def LiveComposite(size, *args, **properties):
     """
@@ -328,6 +233,25 @@ class Position(Container):
 
         return surf
 
+    def get_placement(self):
+    
+        xpos, ypos, xanchor, yanchor, xoffset, yoffset = self.child.get_placement()
+            
+        if self.style.xpos is not None:
+            xpos = self.style.xpos
+
+        if self.style.ypos is not None:
+            ypos = self.style.ypos
+
+        if self.style.xanchor is not None:
+            xanchor = self.style.xanchor
+
+        if self.style.yanchor is not None:
+            yanchor = self.style.yanchor
+
+        return xpos, ypos, xanchor, yanchor, xoffset, yoffset
+
+    
 class Grid(Container):
     """
     A grid is a widget that evenly allocates space to its children.
@@ -346,9 +270,7 @@ class Grid(Container):
         @params transpose: True if the grid should be transposed.
         """
 
-        super(Grid, self).__init__()
-
-        self.style = renpy.style.Style(style, properties)
+        super(Grid, self).__init__(style=style, **properties)
 
         self.cols = cols
         self.rows = rows
@@ -426,6 +348,8 @@ class Grid(Container):
 
 class MultiBox(Container):
 
+    layer_name = None
+    
     def __init__(self, spacing=None, layout=None, style='default', **properties):
 
         if spacing is not None:
@@ -434,23 +358,54 @@ class MultiBox(Container):
         super(MultiBox, self).__init__(style=style, **properties)
 
         self.default_layout = layout
-    
+        
+        self.start_times = [ ]
+        self.anim_times = [ ]
+
+        # A map from layer name to the widget corresponding to
+        # that layer.
+        self.layers = None
+
+        # The scene list for this widget.
+        self.scene_list = None
+        
+        
+    def add(self, widget, start_time=None, anim_time=None):
+        super(MultiBox, self).add(widget)
+        self.start_times.append(start_time)
+        self.anim_times.append(anim_time)
+
+    def append_scene_list(self, l):
+        for tag, zo, start, anim, d in l:
+            self.add(d, start, anim)
+
+        if self.scene_list is None:
+            self.scene_list = [ ]
+            
+        self.scene_list.extend(l)
+        
     def render(self, width, height, st, at):
 
-        layout = self.style.box_layout
-        spacing = self.style.box_spacing
-        first_spacing = self.style.box_first_spacing
-        if first_spacing is None:
-            first_spacing = spacing
+        t = renpy.game.interface.frame_time
+        it = renpy.game.interface.interact_time
 
-        spacings = [ first_spacing ] + [ spacing ] * (len(self.children) - 1)
+        self.start_times = [ i or it for i in self.start_times ]
+        self.anim_times = [ i or it for i in self.anim_times ]
+
+        layout = self.style.box_layout
 
         if layout is None:
             layout = self.default_layout
         
         if layout == "horizontal":
 
-            # This is the horizontal path.
+            spacing = self.style.box_spacing
+            first_spacing = self.style.box_first_spacing
+
+            if first_spacing is None:
+                first_spacing = spacing
+
+            spacings = [ first_spacing ] + [ spacing ] * (len(self.children) - 1)
 
             self.offsets = [ ]
             self.sizes = [ ]
@@ -465,10 +420,10 @@ class MultiBox(Container):
 
             padding = 0
 
-            for i, padding in zip(self.children, spacings):
+            for i, padding, start, anim in zip(self.children, spacings, self.start_times, self.anim_times):
 
                 xoffsets.append(xo)
-                surf = render(i, remwidth, height, st, at)
+                surf = render(i, remwidth, height, t - start, t - anim)
 
                 sw, sh = surf.get_size()
 
@@ -483,22 +438,36 @@ class MultiBox(Container):
                 self.sizes.append((sw, sh))
 
 
-            width = xo - padding
-
             if self.style.yfill:
                 myheight = height
 
+            if not self.style.xfill:
+                width = xo - padding
+                bonus = 0
+            else:
+                bonus = (remwidth + padding) / len(xoffsets)
+                xoffsets = [ xo + i * bonus for i, xo in enumerate(xoffsets) ]
+                
+                
             rv = renpy.display.render.Render(width, myheight)
 
             for surf, child, xo in zip(surfaces, self.children, xoffsets):
                 sw, sh = surf.get_size()
 
-                offset = child.place(rv, xo, 0, sw, myheight, surf)
+                offset = child.place(rv, xo, 0, sw + bonus, myheight, surf)
                 self.offsets.append(offset)
 
             return rv
         
-        else:
+        elif layout == "vertical":
+
+            spacing = self.style.box_spacing
+            first_spacing = self.style.box_first_spacing
+
+            if first_spacing is None:
+                first_spacing = spacing
+
+            spacings = [ first_spacing ] + [ spacing ] * (len(self.children) - 1)
     
             self.offsets = [ ]
             self.sizes = [ ]
@@ -513,11 +482,11 @@ class MultiBox(Container):
 
             padding = 0
 
-            for i, padding in zip(self.children, spacings):
+            for i, padding, start, anim in zip(self.children, spacings, self.start_times, self.anim_times):
 
                 yoffsets.append(yo)
 
-                surf = render(i, width, remheight, st, at)
+                surf = render(i, width, remheight, t - start, t - anim)
 
                 sw, sh = surf.get_size()
 
@@ -531,25 +500,74 @@ class MultiBox(Container):
                 surfaces.append(surf)
                 self.sizes.append((sw, sh))
 
-            height = yo - padding
-
             if self.style.xfill:
                 mywidth = width
 
+            if not self.style.yfill:
+                height = yo - padding
+                bonus = 0
+            else:
+                bonus = (remheight + padding) / len(yoffsets)
+                yoffsets = [ yo + i * bonus for i, yo in enumerate(yoffsets) ]
+                
             rv = renpy.display.render.Render(mywidth, height)
 
             for surf, child, yo in zip(surfaces, self.children, yoffsets):
 
                 sw, sh = surf.get_size()
 
-                offset = child.place(rv, 0, yo, mywidth, sh, surf)
+                offset = child.place(rv, 0, yo, mywidth, sh + bonus, surf)
 
                 self.offsets.append(offset)
 
             return rv
-  
-    
 
+        elif layout == "fixed":
+
+            self.offsets = [ ]
+            self.sizes = [ ]
+            
+            rv = renpy.display.render.Render(width, height, layer_name=self.layer_name)
+
+        
+            for child, start, anim in zip(self.children, self.start_times, self.anim_times):
+
+                cst = t - start
+                cat = t - anim
+
+                surf = render(child, width, height, cst, cat)
+
+                if surf:
+                    self.sizes.append(surf.get_size())
+                    offset = child.place(rv, 0, 0, width, height, surf)
+                    self.offsets.append(offset)
+                else:
+                    self.sizes.append((0, 0))
+                    self.offsets.append((0, 0))
+
+            return rv
+                    
+        
+    def event(self, ev, x, y, st):
+        children_offsets = zip(self.children, self.offsets, self.start_times)
+        children_offsets.reverse()
+
+        for i, (xo, yo), t in children_offsets: 
+
+            if t is None:
+                cst = 0
+            else:
+                cst = renpy.game.interface.event_time - t
+
+            rv = i.event(ev, x - xo, y - yo, cst)    
+            if rv is not None:
+                return rv
+                
+        return None
+
+def Fixed(**properties):
+    return MultiBox(layout='fixed', **properties)
+    
 class Window(Container):
     """
     A window is a container that holds a single Displayable in it. A window
@@ -574,6 +592,9 @@ class Window(Container):
     def visit(self):
         return [ self.style.background ] + self.children
 
+    def get_child(self):
+        return self.style.child or self.child
+    
     def render(self, width, height, st, at):
 
         # save some typing.
@@ -601,8 +622,10 @@ class Window(Container):
         cxpadding = left_padding + right_padding
         cypadding = top_padding + bottom_padding
 
+        child = self.get_child()
+        
         # Render the child.
-        surf = render(self.child,
+        surf = render(child,
                       width  - cxmargin - cxpadding,
                       height - cymargin - cypadding,
                       st, at)
@@ -627,7 +650,7 @@ class Window(Container):
 
             back = render(style.background, bw, bh, st, at)
 
-            style.background.place(rv, left_margin, top_margin, bw, bh, back)
+            style.background.place(rv, left_margin, top_margin, bw, bh, back, main=False)
 
         offsets = self.child.place(rv,
                                    left_margin + left_padding, 
@@ -720,9 +743,9 @@ class Motion(Container):
         self.add_sizes = add_sizes
 
         self.position = (self.style.xpos, self.style.ypos, self.style.xanchor, self.style.yanchor)
-        
+
     def get_placement(self):
-        return self.position
+        return self.position + (self.style.xoffset, self.style.yoffset)
                 
     def render(self, width, height, st, at):
 
@@ -939,7 +962,7 @@ class Revolver(object):
         else:
             pos = self.pos
             
-        xpos, ypos, xanchor, yanchor = pos
+        xpos, ypos, xanchor, yanchor, xoffset, yoffset = pos
 
         xpos = fti(xpos, w)
         ypos = fti(ypos, h)
@@ -1259,16 +1282,16 @@ class DynamicDisplayable(renpy.display.core.Displayable):
             args = ( function, )
             kwargs = { }
             function = dynamic_displayable_compat
-            
+
+        self.predict_function = kwargs.pop("_predict_function", None)            
         self.function = function
         self.args = args
         self.kwargs = kwargs
 
     def visit(self):
-        return [ self.child ]
+        return [ ]
 
     def per_interact(self):
-
         renpy.display.render.redraw(self, 0)
         
     def render(self, w, h, st, at):
@@ -1284,6 +1307,15 @@ class DynamicDisplayable(renpy.display.core.Displayable):
         
         return renpy.display.render.render(self.child, w, h, st, at)
 
+    def predict_one(self, callback):
+
+        if not self.predict_function:
+            return
+        
+        for i in self.predict_function(*self.args, **self.kwargs):
+            if i is not None:
+                i.predict(callback)
+        
     def get_placement(self):
         return self.child.get_placement()
 
@@ -1291,6 +1323,71 @@ class DynamicDisplayable(renpy.display.core.Displayable):
         if self.child:
             return self.child.event(ev, x, y, st)
 
+# This chooses the first member of switch that's being shown on the
+# given layer.
+def condition_switch_pick(switch):
+    for cond, d in switch:
+        if cond is None or renpy.python.py_eval(cond):
+            return d
+
+    raise Excepton("Switch could not choose a displayable.")
+
+def condition_switch_show(st, at, switch):
+    return condition_switch_pick(switch), None
+
+def condition_switch_predict(switch):
+    return [ condition_switch_pick(switch) ]
+
+def ConditionSwitch(*args, **kwargs):
+
+    kwargs.setdefault('style', 'default')
+    
+    switch = [ ]
+    
+    if len(args) % 2 != 0:
+        raise Exception('ConditionSwitch takes an even number of arguments')
+
+    for cond, d in zip(args[0::2], args[1::2]):
+
+        d = renpy.easy.displayable(d)
+        switch.append((cond, d))
+
+    rv = DynamicDisplayable(condition_switch_show,
+                            switch,
+                            _predict_function=condition_switch_predict)
+                              
+    return Position(rv, **kwargs)
+
+    
+def ShowingSwitch(*args, **kwargs):
+
+    layer = kwargs.pop('layer', 'master')
+
+    
+    if len(args) % 2 != 0:
+        raise Exception('ConditionSwitch takes an even number of arguments')
+
+    condargs = [ ]
+
+    
+    for name, d in zip(args[0::2], args[1::2]):
+        if name is not None:
+            if not isinstance(name, tuple):        
+                name = tuple(name.split())
+            cond = "renpy.showing(%r, layer=%r)" % (name, layer)
+        else:
+            cond = None 
+            
+ 
+        condargs.append(cond)
+        condargs.append(d)
+
+    return ConditionSwitch(*condargs, **kwargs)
+    
+    
+    
+
+        
 class IgnoresEvents(renpy.display.core.Displayable):
 
     def __init__(self, child):
@@ -1464,7 +1561,11 @@ class Viewport(Container):
 
         self.child_width, self.child_height = child_size
         self.xoffset, self.yoffset = offsets
-
+#         self.page_height = 0
+#         self.real_child_height = 0
+#         self.page_width = 0
+#         self.real_child_width = 0
+        
     def render(self, width, height, st, at):
 
         child_width = self.child_width or width
@@ -1480,18 +1581,19 @@ class Viewport(Container):
             cxo = max(cw - width, 0) * -self.xoffset
 
         if isinstance(self.yoffset, int):
-            cyo = -self.xoffset
+            cyo = -self.yoffset
         else:
             cyo = max(ch - height, 0) * -self.yoffset 
 
         cxo = int(cxo)
         cyo = int(cyo)
-            
+
         self.offsets = [ (cxo, cyo) ]
         self.sizes = [ (cw, ch) ]
 
         rv = renpy.display.render.Render(width, height)
         rv.blit(surf, (cxo, cyo))
+
         return rv
 
     def set_xoffset(self, offset):
@@ -1505,4 +1607,5 @@ class Viewport(Container):
 def LiveCrop(rect, child, **properties):
     x, y, w, h = rect
 
-    return Viewport(child, offsets=(-x, -y), xmaximum=w, ymaximum=h, **properties)
+    child = renpy.easy.displayable(child)    
+    return Viewport(child, offsets=(x, y), xmaximum=w, ymaximum=h, **properties)
