@@ -162,7 +162,7 @@ def redraw(widget, when):
     Call this to queue the redraw of the supplied widget in the
     supplied number of seconds.
     """
-
+    
     redraw_queue.append((when + renpy.game.interface.frame_time, widget))
     
 def render_screen(widget, width, height, st):
@@ -198,7 +198,7 @@ def render_screen(widget, width, height, st):
 
     # Filter dead widgets from the redraw queue.
     redraw_queue = [ (when, widget) for when, widget in redraw_queue if
-                     widget in live_widgets ]
+                     widget in live_widgets or widget is None]
     
     return rv
 
@@ -495,7 +495,7 @@ class Render(object):
     widget can be withdrawn).
     """
 
-    def __init__(self, width, height, draw_func=None, opaque=False):
+    def __init__(self, width, height, draw_func=None, opaque=False, layer_name=None):
         """
         Creates a new render corresponding to the given widget with
         the specified width and height.
@@ -519,6 +519,10 @@ class Render(object):
         self.children = [ ]
         self.depends = [ ]
 
+        # A list of child renders that were marked important when we
+        # blitted them.
+        self.main_child_renders = [ ]
+        
         # A pygame surface holding this Render, if one exists.
         self.surface = None
         self.surface_alpha = False
@@ -539,6 +543,9 @@ class Render(object):
 
         # Is this render opaque? (Only needs to be set for surfaces with draw_func.)
         self.opaque = opaque
+
+        # The name of this layer, if this render corresponds to a layer.
+        self.layer_name = layer_name
         
     
     # def __del__(self):
@@ -600,6 +607,7 @@ class Render(object):
         # for p in parents:
         #     # assert p.dead
 
+        self.main_child_renders = [ ]
         self.children = [ ]
         self.depends = [ ]
 
@@ -607,7 +615,7 @@ class Render(object):
         self.render_of = [ ]
         self.focuses = [ ]
 
-    def blit(self, source, (xo, yo), focus=True):
+    def blit(self, source, (xo, yo), focus=True, main=True):
         """
         Adds the source to the list of things that need to be blitted
         to the screen. The source should be either a pygame.Surface,
@@ -621,19 +629,25 @@ class Render(object):
             source.parents.append(self)
             self.children.append(source)
 
+            if main:
+                self.main_child_renders.append((xo, yo, source))
+
             if focus and xo == 0 and yo == 0:
                 self.focuses.extend(source.focuses)
             elif focus:
-                for widget, arg, x, y, w, h, mx, my, mask in source.focuses:
-                    if x is not None:
-                        x += xo
-                        y += yo
 
-                    if mx is not None:
-                        mx += xo
-                        my += yo
-                        
-                    self.add_focus(widget, arg, x, y, w, h, mx, my, mask)
+                for f in source.focuses:
+                    nf = f.copy()
+                    
+                    if nf.x is not None:
+                        nf.x += xo
+                        nf.y += yo
+
+                    if nf.mx is not None:
+                        nf.mx += xo
+                        nf.my += yo
+
+                    self.focuses.append(nf)
                                       
         self.blittables.append((xo, yo, source))
     
@@ -809,47 +823,51 @@ class Render(object):
         rv.children.append(self)
 
         if focus:
-            for fwidget, farg, fx, fy, fw, fh, fmx, fmy, fmask in self.focuses:
-                if fx is not None:
-                    fx -= x
-                    fx = max(fx, 0)
-                    fy -= y
-                    fy = max(fy, 0)
 
-                    fw -= x
-                    fw = min(fw, width)
-                    fh -= y
-                    fh = min(fh, height)
+            for f in self.focuses:
 
-                    if fw <= 0 or fh <= 0:
+                nf = f.copy()
+
+                if nf.x is not None:
+                    nf.x -= x
+                    nf.x = max(nf.x, 0)
+                    nf.y -= y
+                    nf.y = max(nf.y, 0)
+
+                    nf.w -= x
+                    nf.w = min(nf.w, width)
+                    nf.h -= y
+                    nf.h = min(nf.h, height)
+
+                    if nf.w <= 0 or nf.h <= 0:
                         continue
 
-                if fmx is not None:
+                if nf.mx is not None:
 
-                    fmx -= x
-                    if fmx < 0:
+                    nf.mx -= x
+                    if nf.mx < 0:
                         fmxo = -fmx
-                        fmx = 0
+                        nf.mx = 0
                     else:
                         fmxo = 0
 
-                    fmy -= y
-                    if fmy < 0:
+                    nf.my -= y
+                    if nf.my < 0:
                         fmyo = -fmy
-                        fmy = 0
+                        nf.my = 0
                     else:
                         fmyo = 0
                     
-                    fmw, fmh = fmask.get_size()
-                    fmw = min(fmw - fmxo, width - fmx)
-                    fmh = min(fmh - fmyo, height - fmy)
+                    fmw, fmh = nf.mask.get_size()
+                    fmw = min(fmw - fmxo, width - nf.mx)
+                    fmh = min(fmh - fmyo, height - nf.my)
 
                     if fmw <= 0 or fmh <= 0:
                         continue
 
-                    fmask = fmask.subsurface((fmxo, fmyo, fmw, fmh))
+                    nf.mask = nf.mask.subsurface((fmxo, fmyo, fmw, fmh))
                     
-                rv.add_focus(fwidget, farg, fx, fy, fw, fh, fmx, fmy, fmask)
+                rv.focuses.append(nf)
                 
         for xo, yo, source in self.blittables:
 
@@ -963,7 +981,27 @@ class Render(object):
 
         return False
                 
+    def main_displayables_at_point(self, x, y, layers, depth=None):
         
+        if x < 0 or y < 0 or x >= self.width or y >= self.height:
+            return [ ]
+
+        rv = [ ]
+        
+        if depth is not None:
+            for w, width, height in self.render_of:
+                rv.append((depth, self.width, self.height, w))
+
+            depth += 1
+            
+        elif self.layer_name in layers:
+            depth = 0
+
+        for xo, yo, child in self.main_child_renders:
+            rv.extend(child.main_displayables_at_point(x - xo, y - yo, layers, depth))
+
+        return rv
+            
         
         
 def is_fullscreen(surf, x, y, wh):
