@@ -585,11 +585,6 @@ def subtitle_text_layout_core(triples, width, style, soft, n):
 
         type, ts, i = triple
 
-        if type == "newline":
-            raise Exception("Subtitles may not contain newlines.")
-
-            continue
-
         if type == "space":
             if not line:
                 continue
@@ -597,9 +592,10 @@ def subtitle_text_layout_core(triples, width, style, soft, n):
             continue
 
         else:
+
             lw = layout_width(line + [ triple ])
 
-            if lw > target:
+            if lw > target or type == "newline":
 
                 n -= 1
                 if n > 0:
@@ -613,8 +609,13 @@ def subtitle_text_layout_core(triples, width, style, soft, n):
                     linesoft = soft
 
                 lines.append(line)
-                line = [ triple ]
                 target = width
+
+                if type == "newline":
+                    line = [ ]
+                else:
+                    line = [ triple ]
+
             else:
                 line.append(triple)
 
@@ -687,7 +688,8 @@ class Text(renpy.display.core.Displayable):
         self._update()
 
     def __init__(self, text, slow=None, slow_done=None,
-                 slow_start=0, pause=None, tokenized=False, style='default', **properties):
+                 slow_start=0, pause=None, tokenized=False,
+                 style='default', **properties):
         """
         @param text: The text that will be displayed on the screen.
 
@@ -707,17 +709,14 @@ class Text(renpy.display.core.Displayable):
 
         """
 
-        super(Text, self).__init__()
+        super(Text, self).__init__(style=style, **properties)
 
         self.text = text        
-        self.style = renpy.style.Style(style, properties)
         self.pause = pause
         self.keep_pausing = False
         self.pause_length = None
         self.tokenized = tokenized
 
-        self._update(redraw=False)
-
         if slow or slow is None:
             self.slow = True
         else:
@@ -727,9 +726,10 @@ class Text(renpy.display.core.Displayable):
         self.slow_done = slow_done
         self.slow_start = slow_start
 
-
         self.laidout = None
         self.child_pos = [ ]
+        
+        self._update(redraw=False)
 
 
     def set_text(self, new_text):
@@ -831,7 +831,10 @@ class Text(renpy.display.core.Displayable):
                             pause -= 1
 
             self.tokens[0] = new_tokens
-        
+
+        if not self.slow:
+            self.no_wait = False
+            self.no_wait_once = False
             
         # Postprocess the tokens list to create widgets, as necessary.
 
