@@ -24,6 +24,9 @@
 # their behavior, while functions imported in are probably best left
 # alone as part of the api.
 
+# Remember the real file.
+_file = file
+
 import renpy
 
 # Many of these shouldn't be used directly.
@@ -37,7 +40,7 @@ from renpy.curry import curry
 # from renpy.display.audio import music_start, music_stop
 from renpy.audio.sound import play
 from renpy.display.video import movie_start_fullscreen, movie_start_displayable, movie_stop
-from renpy.loadsave import load, save, list_saved_games, can_load, rename_save, unlink_save
+from renpy.loadsave import load, save, list_saved_games, can_load, rename_save, unlink_save, scan_saved_game
 from renpy.python import py_eval as eval
 from renpy.python import rng as random
 
@@ -144,10 +147,7 @@ def showing(name, layer='master'):
     if not isinstance(name, tuple):
         name = tuple(name.split())
 
-    sls = scene_lists()
-    key = name[0]
-
-    return sls.showing(layer, key)
+    return renpy.game.context().predict_info.images.showing(layer, name)
 
 def show(name, at_list=[ ], layer='master', what=None, zorder=0, tag=None, behind=[ ]):
     "Documented in wiki as renpy.show."
@@ -160,10 +160,8 @@ def show(name, at_list=[ ], layer='master', what=None, zorder=0, tag=None, behin
     key = tag or name[0]
 
     if renpy.config.sticky_positions:        
-        if not at_list and key in sls.sticky_positions:
-            at_list = sls.sticky_positions[key]
-
-        sls.sticky_positions[key] = at_list
+        if not at_list and key in sls.at_list[layer]:
+            at_list = sls.at_list[layer][key]
 
     if what is None:
         what = name
@@ -173,9 +171,12 @@ def show(name, at_list=[ ], layer='master', what=None, zorder=0, tag=None, behin
         img = i(img)
 
     # Update the list of images we have ever seen.
-    renpy.game.persistent._seen_images[tuple(name)] = True
+    renpy.game.persistent._seen_images[name] = True
 
-    sls.add(layer, img, key, zorder, behind)
+    if tag:
+        name = (tag,) + name[1:]
+    
+    sls.add(layer, img, key, zorder, behind, at_list=at_list, name=name)
     
 
 def hide(name, layer='master'):
@@ -199,10 +200,6 @@ def hide(name, layer='master'):
     sls = scene_lists()
     key = name[0]
     sls.remove(layer, key)
-
-    if key in sls.sticky_positions:
-        del sls.sticky_positions[key]
-
     
 
 def scene(layer='master'):
@@ -214,7 +211,6 @@ def scene(layer='master'):
 
     sls = scene_lists()
     sls.clear(layer)
-    sls.sticky_positions.clear()
     
         
 def watch(expression, style='default', **properties):
@@ -564,7 +560,7 @@ def pause(delay=None, music=None, with_none=None, hard=False):
     return rv
 
 
-def movie_cutscene(filename, delay, loops=0):
+def movie_cutscene(filename, delay=None, loops=0):
     """
     This displays an MPEG-1 cutscene for the specified number of
     seconds. The user can click to interrupt the cutscene.
@@ -572,8 +568,11 @@ def movie_cutscene(filename, delay, loops=0):
 
     @param filename: The name of a file containing an MPEG-1 movie.
 
-    @param delay: The number of seconds to wait before ending the cutscene. Normally the length of the movie, in seconds.
-
+    @param delay: The number of seconds to wait before ending the cutscene.
+    Normally the length of the movie, in seconds. If None, then the
+    delay is computed from the number of loops (that is, loops + 1) *
+    the length of the movie. If -1, we wait until the user clicks.
+    
     @param loops: The number of extra loops to show, -1 to loop forever.
 
     Returns True if the movie was terminated by the user, or False if the
@@ -583,7 +582,15 @@ def movie_cutscene(filename, delay, loops=0):
     movie_start_fullscreen(filename, loops=loops)
 
     renpy.ui.saybehavior()
-    renpy.ui.pausebehavior(delay, False)
+
+    if delay is None:
+        if loops < 0:
+            delay = -1
+        else:
+            delay = renpy.display.video.movie_length(filename) * (loops + 1)
+
+    if delay >= 0:
+        renpy.ui.pausebehavior(delay, False)
 
     if renpy.game.log.forward:
         roll_forward = True
@@ -935,7 +942,7 @@ def log(msg):
     if not logfile:
         import codecs
 
-        logfile = file(renpy.config.log, "a")
+        logfile = _file(renpy.config.log, "a")
         if not logfile.tell():
             logfile.write(codecs.BOM_UTF8)
 
@@ -964,6 +971,36 @@ def last_interact_type():
 def dynamic(*vars):
     renpy.game.context().make_dynamic(vars)
 
+def seen_label(label):
+    return label in renpy.game.seen_ever
+
+def file(fn):
+    return renpy.loader.load(fn)
+
+def image_size(im):
+    # Index the archives, if we haven't already.
+    renpy.loader.index_archives()
+
+    im = renpy.easy.displayable(im)
+
+    if not isinstance(im, renpy.display.im.Image):
+        raise Exception("renpy.image_size expects it's argument to be an image.")
+
+    surf = im.load()
+    return im.get_size()
+
+def get_at_list(name, layer='master'):
+    if isinstance(name, basestring):
+        name = tuple(name.split())
+
+    tag = name[0]
+
+    return renpy.game.context().scene_lists.at_list[layer].get(tag, None)
+
+def layer_at_list(at_list, layer='master'):
+    renpy.game.context().scene_lists.set_layer_at_list(layer, at_list)
+
+    
 # New context stuff.
 call_in_new_context = renpy.game.call_in_new_context
 curried_call_in_new_context = renpy.curry.curry(renpy.game.call_in_new_context)
