@@ -86,6 +86,9 @@ def list_logical_lines(filename):
     data = f.read()
     f.close()
 
+    data = data.replace("\r\n", "\n")
+    data = data.replace("\r", "\n")
+    
     if "RENPY_PATH_ELIDE" in os.environ:
         old, new = os.environ["RENPY_PATH_ELIDE"].split(':')
         filename = filename.replace(old, new)
