@@ -81,13 +81,15 @@ def add(w, make_current=False, once=False):
     global current
     global current_once
 
+    atw = w 
+    
     while at_stack:
-        w = at_stack.pop()(w)
+        atw = at_stack.pop()(atw)
     
     if isinstance(current, str):
-        renpy.game.context(-1).scene_lists.add(current, w)
+        renpy.game.context(-1).scene_lists.add(current, atw)
     else:
-        current.add(w)
+        current.add(atw)
 
     if current_once:
         current_once = False
@@ -261,7 +263,7 @@ def fixed(**properties):
     widget.
     """
 
-    rv = renpy.display.layout.Fixed(**properties)
+    rv = renpy.display.layout.MultiBox(layout='fixed', **properties)
     add(rv, True)
 
     return rv
@@ -399,7 +401,10 @@ def menu(menuitems,
             else:
                 def clicked(val=val):
                     return val
-            
+
+            button = getattr(renpy.game.style, button)[label]
+            text = getattr(renpy.game.style, text)[label]
+                
             renpy.ui.textbutton(label,
                                 style=button,
                                 text_style=text,
@@ -455,12 +460,15 @@ def imagemap(ground, selected, hotspots, unselected=None,
     image(ground)
 
     for x0, y0, x1, y1, result in hotspots:
-        imagebutton(renpy.display.im.Crop(unselected, x0, y0, x1 - x0, y1 - y0),
-                    renpy.display.im.Crop(selected, x0, y0, x1 - x0, y1 - y0),
+
+        imagebutton(renpy.display.layout.LiveCrop((x0, y0, x1 - x0, y1 - y0), unselected),
+                    renpy.display.layout.LiveCrop((x0, y0, x1 - x0, y1 - y0), selected),
                     clicked=returns(result),
                     style=button_style,
-                    xpos=x0, xanchor='left',
-                    ypos=y0, yanchor='top',
+                    xpos=x0,
+                    xanchor=0,
+                    ypos=y0,
+                    yanchor=0,
                     focus_mask=True,
                     )
 
@@ -518,8 +526,18 @@ def textbutton(text, clicked=None, text_style='button_text', **properties):
                                                  text_style=text_style,
                                                  **properties))
 
-def imagebutton(idle_image, hover_image, clicked=None,
-                image_style='image_button_image', **properties):
+def imagebutton(idle_image,
+                hover_image,
+                insensitive_image = None,
+                activate_image = None,
+                selected_idle_image = None,
+                selected_hover_image = None,
+                selected_insensitive_image = None,
+                selected_activate_image = None,    
+                clicked=None,
+                style='image_button',
+                image_style=None,
+                **properties):
 
     """
     This creates a button that contains two images. The first is the
@@ -538,18 +556,22 @@ def imagebutton(idle_image, hover_image, clicked=None,
     @param clicked: The function that is called when this button is
     clicked.
 
-    @param image_style: The style that is applied to the images that
-    are used as part of the imagebutton.
-
     @param role: The role this button undertakes. This can be the empty
     string, or "selected_".
     """
     
-    return add(renpy.display.image.ImageButton(idle_image,
-                                               hover_image,
-                                               clicked=clicked,
-                                               image_style=image_style,
-                                               **properties))
+    return add(renpy.display.image.ImageButton(
+            idle_image,
+            hover_image,
+            insensitive_image = insensitive_image,
+            activate_image = activate_image,
+            selected_idle_image = selected_idle_image,
+            selected_hover_image = selected_hover_image,
+            selected_insensitive_image = selected_insensitive_image,
+            selected_activate_image = selected_activate_image,    
+            clicked=clicked,
+            style=style,
+            **properties))
 
 def bar(*args, **properties):
     """
